@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2013	PgPool Global Development Group
+ * Copyright (c) 2003-2015	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -100,6 +100,7 @@ void pool_start_query(POOL_QUERY_CONTEXT *query_context, char *query, int len, N
 		query_context->parse_tree = node;
 		query_context->virtual_master_node_id = my_master_node_id;
 		query_context->is_cache_safe = false;
+		query_context->num_original_params = -1;
 		if (pool_config->memory_cache_enabled)
 			query_context->temp_cache = pool_create_temp_query_cache(query);
 		pool_set_query_in_progress();
@@ -118,7 +119,7 @@ void pool_set_node_to_be_sent(POOL_QUERY_CONTEXT *query_context, int node_id)
 		return;
 	}
 
-	if (node_id < 0 || node_id > MAX_NUM_BACKENDS)
+	if (node_id < 0 || node_id >= MAX_NUM_BACKENDS)
 	{
 		pool_error("pool_set_node_to_be_sent: invalid node id:%d", node_id);
 		return;
@@ -140,7 +141,7 @@ void pool_unset_node_to_be_sent(POOL_QUERY_CONTEXT *query_context, int node_id)
 		return;
 	}
 
-	if (node_id < 0 || node_id > MAX_NUM_BACKENDS)
+	if (node_id < 0 || node_id >= MAX_NUM_BACKENDS)
 	{
 		pool_error("pool_unset_node_to_be_sent: invalid node id:%d", node_id);
 		return;
@@ -171,6 +172,14 @@ void pool_clear_node_to_be_sent(POOL_QUERY_CONTEXT *query_context)
 void pool_setall_node_to_be_sent(POOL_QUERY_CONTEXT *query_context)
 {
 	int i;
+	POOL_SESSION_CONTEXT *sc;
+
+	sc = pool_get_session_context();
+	if (!sc)
+	{
+		pool_error("pool_setall_node_to_be_sent: no session context");
+		return;
+	}
 
 	if (!query_context)
 	{
@@ -182,7 +191,20 @@ void pool_setall_node_to_be_sent(POOL_QUERY_CONTEXT *query_context)
 	{
 		if (private_backend_status[i] == CON_UP ||
 			(private_backend_status[i] == CON_CONNECT_WAIT))
+		{
+			/*
+			 * In streaming replication mode, if the node is not
+			 * primary node nor load balance node, there's no point to
+			 * send query.
+			 */
+			if (pool_config->master_slave_mode &&
+				!strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP) &&
+				i != PRIMARY_NODE_ID && i != sc->load_balance_node_id)
+			{
+				continue;
+			}
 			query_context->where_to_send[i] = true;
+		}
 	}
 	return;
 }
@@ -228,7 +250,7 @@ bool pool_is_node_to_be_sent(POOL_QUERY_CONTEXT *query_context, int node_id)
 		return false;
 	}
 
-	if (node_id < 0 || node_id > MAX_NUM_BACKENDS)
+	if (node_id < 0 || node_id >= MAX_NUM_BACKENDS)
 	{
 		pool_error("pool_is_node_to_be_sent: invalid node id:%d", node_id);
 		return false;
@@ -516,7 +538,8 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 		}
 		else
 		{
-			if (is_select_query(node, query) && !pool_config->replicate_select)
+			if (is_select_query(node, query) && !pool_config->replicate_select &&
+				!pool_has_function_call(node))
 			{
 				/* only send to master node */
 				pool_set_node_to_be_sent(query_context, REAL_MASTER_NODE_ID);
@@ -1057,6 +1080,19 @@ static POOL_DEST send_to_where(Node *node, char *query)
 			if (pool_has_insertinto_or_locking_clause(node))
 				return POOL_PRIMARY;
 
+			/* non-SELECT query in WITH clause ? */
+			if (((SelectStmt *)node)->withClause)
+			{
+				List *ctes = ((SelectStmt *)node)->withClause->ctes;
+				ListCell   *cte_item;
+				foreach(cte_item, ctes)
+				{
+					CommonTableExpr *cte = (CommonTableExpr *)lfirst(cte_item);
+					if (!IsA(cte->ctequery, SelectStmt))
+						return POOL_PRIMARY;
+				}
+			}
+
 			return POOL_EITHER;
 		}
 
@@ -1250,11 +1286,10 @@ void where_to_send_deallocate(POOL_QUERY_CONTEXT *query_context, Node *node)
 	DeallocateStmt *d = (DeallocateStmt *)node;
 	POOL_SENT_MESSAGE *msg;
 
-	/* DELLOCATE ALL? */
+	/* DEALLOCATE ALL? */
 	if (d->name == NULL)
 	{
 		pool_setall_node_to_be_sent(query_context);
-		return;
 	}
 	else
 	{
@@ -1266,11 +1301,11 @@ void where_to_send_deallocate(POOL_QUERY_CONTEXT *query_context, Node *node)
 			/* Inherit same map from PREPARE or PARSE */
 			pool_copy_prep_where(msg->query_context->where_to_send,
 								 query_context->where_to_send);
+			return;
 		}
-		return;
+		/* prepared statement was not found */
+		pool_setall_node_to_be_sent(query_context);
 	}
-	/* prepared statement was not found */
-	pool_setall_node_to_be_sent(query_context);
 }
 
 /*
@@ -1679,3 +1714,77 @@ void pool_unset_cache_exceeded(void)
 		sc->query_context->temp_cache->is_exceeded = false;
 	}
 }
+
+/*
+ * Return true if one of followings is true
+ *
+ * SET transaction_read_only TO on
+ * SET TRANSACTION READ ONLY
+ * SET TRANSACTION CHARACTERISTICS AS TRANSACTION READ ONLY
+ *
+ * Note that if the node is not a variable statement, returns false.
+ */
+bool pool_is_transaction_read_only(Node *node)
+{
+	ListCell   *list_item;
+	bool ret = false;
+
+	if (!IsA(node, VariableSetStmt))
+		return ret;
+
+	/*
+	 * SET transaction_read_only TO on
+	 */
+	if (((VariableSetStmt *)node)->kind == VAR_SET_VALUE &&
+		!strcmp(((VariableSetStmt *)node)->name, "transaction_read_only"))
+	{
+		List *options = ((VariableSetStmt *)node)->args;
+		foreach(list_item, options)
+		{
+			A_Const *v = (A_Const *)lfirst(list_item);
+
+			switch (v->val.type)
+			{
+				case T_String:
+					if (!strcasecmp(v->val.val.str, "on") ||
+						!strcasecmp(v->val.val.str, "t") ||
+						!strcasecmp(v->val.val.str, "true"))
+						ret = true;
+					break;
+				case T_Integer:
+					if (v->val.val.ival)
+						ret = true;
+				default:
+					break;
+			}
+		}
+	}
+
+	/*
+	 * SET SESSION CHARACTERISTICS AS TRANSACTION READ ONLY
+	 * SET TRANSACTION READ ONLY
+	 */
+	else if (((VariableSetStmt *)node)->kind == VAR_SET_MULTI &&
+			 (!strcmp(((VariableSetStmt *)node)->name, "TRANSACTION") ||
+			  !strcmp(((VariableSetStmt *)node)->name, "SESSION CHARACTERISTICS")))
+	{
+		List *options = ((VariableSetStmt *)node)->args;
+		foreach(list_item, options)
+		{
+			DefElem *opt = (DefElem *) lfirst(list_item);
+
+			if (!strcmp("transaction_read_only", opt->defname))
+			{
+				bool read_only;
+
+				read_only = ((A_Const *)opt->arg)->val.val.ival;
+				if (read_only)
+				{
+					ret = true;
+					break;
+				}
+			}
+		}
+	}
+	return ret;
+}
