@@ -169,7 +169,6 @@ exec_ifconfig(char * path,char * command)
 	int pid, i = 0;
 	char* buf;
 	char *bp, *ep;
-	sighandler_t sig_org;
 
 	if (pipe(pfd) == -1)
 	{
@@ -207,8 +206,6 @@ exec_ifconfig(char * path,char * command)
 	}
 	args[i++] = NULL;
 
-	sig_org = signal(SIGCHLD, SIG_DFL);
-
 	pid = fork();
 	if (pid == 0)
 	{
@@ -225,14 +222,15 @@ exec_ifconfig(char * path,char * command)
 		for (;;)
 		{
 			int result;
+
 			result = waitpid(pid, &status, 0);
-			if (result < 0 && errno != ECHILD)
+			if (result < 0)
 			{
 				if (errno == EINTR)
 					continue;
 
 				pool_debug("exec_ifconfig: wait() failed. reason: %s ", strerror(errno));
-				signal(SIGCHLD, sig_org);
+
 				return WD_NG;
 			}
 
@@ -240,7 +238,6 @@ exec_ifconfig(char * path,char * command)
 			{
 				pool_debug("exec_ifconfig: '%s' failed. exit status: %d",
 				           command, WEXITSTATUS(status));
-				signal(SIGCHLD, sig_org);
 				return WD_NG;
 			}
 			else
@@ -250,7 +247,7 @@ exec_ifconfig(char * path,char * command)
 	}
 
 	pool_debug("exec_ifconfig: '%s' succeeded", command);
-	signal(SIGCHLD, sig_org);
+
 	return WD_OK;
 }
 
